using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using BusinessObjects.Enterprise;
using BusinessObjects.Enterprise.Framework;
using BusinessObjects.Enterprise.Infostore;


namespace BusinessObjects_SDK_Tutorial
{
  public enum BObjTypes
  {
    Calendar,
    Category,
    Connection,
    CrystalReport,
    Diskunmanaged,
    Event,
    Excel,
    FavoritesFolder,
    Folder,
    Ftp,
    Hyperlink,
    Inbox,
    Managed,
    ObjectPackage,
    Overload,
    PersonalCategory,
    Pdf,
    Powerpoint,
    Profile,
    Program,
    privateation,
    Rtf,
    Shortcut,
    Smtp,
    Txt,
    Universe,
    User,
    Usergroup,
    Webi,
    Word
  }
  
  public partial class SDK_Tutorial_Form : Form
  {
    private IEnterpriseSession _BOESession;
    private bool _initialized = false;
    private const string C_NUMBERS = "0123456789";
    private IInfoStore _BOEInfoStore;

    public SDK_Tutorial_Form()
    {
      InitializeComponent();
    }

    private void SDK_Tutorial_Form_FormClosing(object sender, FormClosingEventArgs e)
    {
      if (_initialized)
      {
        _BOESession.Logoff();
      }
    }

    /// <summary>
    /// Log in to the CMS
    /// </summary>
    private void cmsBtn_Click(object sender, EventArgs e)
    {
      if ((cmsUser.Text == string.Empty) || (cmsName.Text == string.Empty))
      {
        MessageBox.Show("Unable to log in to BOE - User ID or CMS Name not set",
                        "Login Failure", MessageBoxButtons.OK);
      }
      if (_initialized)
      {
        //if we're already logged in to a CMS, log off first
        _BOESession.Logoff();
        _initialized = false;
        cmsStatus.Text = "NOT Logged in to CMS!";
        cmsStatus.Refresh();
      }
      //log in to CMS
      ISessionMgr sessionMgr = CrystalEnterprise.GetSessionMgr();
      try
      {
        cmsStatus.Text = "Logging in to CMS...";
        cmsStatus.Refresh();
        _BOESession = sessionMgr.Logon(cmsUser.Text, cmsPW.Text, cmsName.Text, "secEnterprise");
        _BOEInfoStore = (IInfoStore)_BOESession.GetService("InfoStore");
        _initialized = true;
        cmsStatus.Text = "Logged in to CMS " + cmsName.Text + ".";
        cmsStatus.Refresh();
        tutorialTabs.Enabled = true;
      }
      catch (Exception ex)
      {
        MessageBox.Show("Unable to log in to BOE - " + ex.Message);
        cmsStatus.Text = "Login Failed";
      }
    }
    
    #region common

    private bool IsNumber(string txt)
    {
      int i = 0;
      bool result = (txt != string.Empty);
      while ((i <= txt.Length - 1) && (result))
      {
        result = (C_NUMBERS.IndexOf(txt[i]) >= 0);
        i++;
      }
      return result;
    }

    /// <summary>
    /// Get an object's ID based on its name
    /// </summary>
    /// <param name="objectName"> The object name to serch for</param>
    /// <returns></returns>
    private int GetIDFromName(string objectName)
    {
      int result = -1;
      //get the ID of the Object
      string query = "Select SI_ID"
        + " From CI_INFOOBJECTS "
        + " Where SI_KIND in ('Folder', 'Report')"
        + "   and SI_NAME='" + objectName + "'";
      //"using" ensures that the object is implicitly disposed when it goes out of scope
      IInfoObjects infoObjects = _BOEInfoStore.Query(query);
      
        if (infoObjects.Count > 0)
        {
          if (infoObjects.Count == 1)
            result = infoObjects[1].ID;
          else
          {
            MessageBox.Show(objectName + " occurs multiple times in the CMS");
          }
        }
      
      return result;
    }

    /// <summary>
    /// Get an object's ID based on its name and its parent's ID.
    /// This is used when there are multiple objects with the same name
    /// in different folders and you need to make sure you have the right one.
    /// </summary>
    /// <param name="objectName">Name of the object to search for</param>
    /// <param name="parentID">ID of the object's parent folder</param>
    /// <returns></returns>
    private int GetIDFromName(string objectName, int parentID)
    {
      int result = -1;
      //get the ID of the Object
      string query = "Select SI_ID"
        + " From CI_INFOOBJECTS "
        + " Where SI_NAME='" + objectName + "'"
        + "   and SI_PARENTID=" + parentID.ToString();

      //"using" ensures that the object is implicitly disposed when it goes out of scope
      IInfoObjects infoObjects = _BOEInfoStore.Query(query);
      
        if (infoObjects.Count > 0)
          result = infoObjects[1].ID;
      
      return result;
    }

    /// <summary>
    /// Get the name of an object based on its ID
    /// </summary>
    /// <param name="id"> The object ID to search for</param>
    /// <returns></returns>
    private string GetNameFromID(int objectId)
    {
      string result = "Not Found";
      //get the ID of the Object
      string query = "Select SI_NAME"
        + " From CI_INFOOBJECTS "
        + " Where SI_ID=" + objectId.ToString();

      IInfoObjects infoObjects = _BOEInfoStore.Query(query);
      
        if (infoObjects.Count > 0)
          result = infoObjects[1].Title;
      
      return result;
    }

    #endregion common

    #region general_tab_methods
    private void findID_Click(object sender, EventArgs e)
    {
      if (getIDParentID.Text == string.Empty)
      {
        getIDLabel.Text = "ID: " + GetIDFromName(getIDObjectName.Text);
      }
      else if (IsNumber(getIDParentID.Text))
      {
        getIDLabel.Text = "ID: " + GetIDFromName(getIDObjectName.Text, Convert.ToInt32(getIDParentID.Text));
      }
      else
      {
        MessageBox.Show("Parent ID MUST be blank or a number!", "Error", MessageBoxButtons.OK);
      }
    }

    private void findName_Click(object sender, EventArgs e)
    {
      if (IsNumber(getNameObjectID.Text))
      {
        getNameLabel.Text = "Name: " + GetNameFromID(Convert.ToInt32(getNameObjectID.Text));
      }
      else
      {
        MessageBox.Show("Object ID MUST be a number!", "Error", MessageBoxButtons.OK);
      }
    }

    #endregion general_tab_methods

    #region folder_list_methods

    private void getFolders_Click(object sender, EventArgs e)
    {
      //this may take a few seconds so we set the cursor to wait
      Cursor = Cursors.WaitCursor;
      //Disable updates in the list box so that it doesn't refresh with every entry.
      //This speeds up processing when GUI controls are involved;
      folderListBox.BeginUpdate();
      try
      {
        List<BOEFolderInfo> folderList = new List<BOEFolderInfo>();
        folderListBox.Items.Clear();
        //Get the folder list from BusinessObjects
        if (IsNumber(folderListParentID.Text))
        {
          folderList = GetFolderList(Convert.ToInt32(folderListParentID.Text));
        }
        else if (folderListParentName.Text != string.Empty)
        {
          folderList = GetFolderList(folderListParentName.Text);
        }
        else
        {
          MessageBox.Show("Parent ID or Parent Name MUST contain a valid entry!", "Error", MessageBoxButtons.OK);
        }
        //Put the folder names in the list box
        if (folderList.Count > 0)
        {
          foreach (BOEFolderInfo folder in folderList)
          {
            folderListBox.Items.Add("Name=" + folder.FolderName + "; ID=" + folder.ID.ToString());
          }
        }
      }
      finally
      {
        //Re-enable updates in the list box which will automatically refresh with the new info
        folderListBox.EndUpdate();
        //reset the cursor
        Cursor = Cursors.Default;
      }
    }
 
    /// <summary>
    /// Get an array of BOEFolderInfo objects for the subfolders of the
    /// given folder based on the parent folder's name.
    /// </summary>
    /// <param name="parentName"> Name of the parent folder</param>
    /// <returns></returns>
    private List<BOEFolderInfo> GetFolderList(string parentName)
    {
      int startFolder = GetIDFromName(parentName);
      List<BOEFolderInfo> result = GetFolderList(startFolder);
      return result;
    }

    /// <summary>
    /// Get an array of BOEFolderInfo objects for the subfolders of the
    /// given folder based on the parent folder's id.
    /// </summary>
    /// <param name="parentID"> ID of the parent folder</param>
    /// <returns></returns>
    private List<BOEFolderInfo> GetFolderList(int parentID)
    {
      List<BOEFolderInfo> folderList = new List<BOEFolderInfo>();
      string query = "Select SI_NAME, SI_ID, SI_PARENTID, SI_PATH"
        + " From CI_INFOOBJECTS "
        + " Where SI_KIND='Folder'"
        + " And SI_PARENT_FOLDER=" + parentID.ToString()
        + " Order By SI_NAME";

      //"using" ensures that the object is implicitly disposed when it goes out of scope
      IInfoObjects infoObjects = _BOEInfoStore.Query(query);
      
        IInfoObject folder;
        for (int i = 1; i <= infoObjects.Count; i++)
        {
          folder = (IInfoObject)infoObjects[i];
          folderList.Add(new BOEFolderInfo(folder));
        }
      
      return folderList;
    }


    #endregion folder_list_methods

    #region report_list_methods
    private void getRpts_Click(object sender, EventArgs e)
    {
      //this may take a few seconds so we set the cursor to wait
      Cursor = Cursors.WaitCursor;
      //Disable updates in the list box so that it doesn't refresh with every entry.
      //This speeds up processing when GUI controls are involved;
      rptListBox.BeginUpdate();
      try
      {
        List<BOERptInfo> rptList = new List<BOERptInfo>();
        folderListBox.Items.Clear();
        //Get the folder list from BusinessObjects
        if (IsNumber(getRptParentID.Text))
        {
          rptList = GetReportsInFolder(Convert.ToInt32(getRptParentID.Text));
        }
        else if (getRptParentName.Text != string.Empty)
        {
          Int32 parentID = GetIDFromName(folderListParentName.Text);
          if (parentID >= 0)
          {
            rptList = GetReportsInFolder(parentID);
          }
          else
          {
            MessageBox.Show("ID not found for Parent Name", "Error", MessageBoxButtons.OK);
          }
        }
        else
        {
          MessageBox.Show("Parent ID or Parent Name MUST contain a valid entry!", "Error", MessageBoxButtons.OK);
        }
        //Put the folder names in the list box
        if (rptList.Count > 0)
        {
          foreach (BOERptInfo rpt in rptList)
          {
            rptListBox.Items.Add("Name=" + rpt.RptName + "; ID=" + rpt.ID.ToString());
          }
        }
      }
      finally
      {
        //Re-enable updates in the list box which will automatically refresh with the new info
        rptListBox.EndUpdate();
        //reset the cursor
        Cursor = Cursors.Default;
      }
    }

    /// <summary>
    /// Get a list of all of the Crystal reports in a folder.  
    /// At a later date this will probably be expanded to include WebI reports.
    /// </summary>
    /// <param name="folderID"> The ID of the folder we're looking at</param>
    /// <returns></returns>
    private List<BOERptInfo> GetReportsInFolder(int folderID)
    {
      List<BOERptInfo> reports = new List<BOERptInfo>();
      string query = "SELECT SI_NAME, SI_ID, SI_KIND "
                     + "FROM CI_INFOOBJECTS "
                     + "WHERE SI_PARENTID = " + folderID.ToString()
                     + " and (SI_KIND = 'CrystalReport') "
                     + "ORDER BY SI_NAME";

      IInfoObjects infoObjects = _BOEInfoStore.Query(query);
      
        IInfoObject report;
        for (int i = 1; i <= infoObjects.Count; i++)
        {
          report = infoObjects[i];
          reports.Add(new BOERptInfo(report));
        }
      
      return reports;
    }

    #endregion report_list_methods

    #region tree_view_methods
    private void getTree_Click(object sender, EventArgs e)
    {
      //this may take a few seconds so we set the cursor to wait
      Cursor = Cursors.WaitCursor;
      //Disable updates in the tree view so that it doesn't refresh with every entry.
      //This speeds up processing when GUI controls are involved;
      folderTreeView.Nodes.Clear();
      folderTreeView.BeginUpdate();
      treeStatusLbl.Text = "Getting folder info...";
      treeStatusLbl.Refresh();
      try
      {
        List<BOEFolderInfo> treeList;
        string topNodeName = string.Empty;
        Int32 topNodeID = -1;
        Int32 topNodeParentID = -1;
        //get base list containing the folder tree and set up data for the top node
        if (IsNumber(getTreeFolderID.Text))
        {
          //working from an ID
          topNodeID = Convert.ToInt32(getTreeFolderID.Text);
          treeList = GetFolderTree(topNodeID);
          topNodeName = GetNameFromID(topNodeID);
          topNodeParentID = GetParentID(topNodeID);
        }
        else if (getTreeFolderName.Text != string.Empty)
        {
          //working from a folder name
          treeList = GetFolderTree(getTreeFolderName.Text);
          topNodeName = getTreeFolderName.Text;
          topNodeID = GetIDFromName(getTreeFolderName.Text);
          topNodeParentID = GetParentID(topNodeID);
        }
        else
        {
          MessageBox.Show("Parent ID or Parent Name MUST contain a valid entry!", "Error", MessageBoxButtons.OK);
          treeList = new List<BOEFolderInfo>();  //initialize empty list so we don't get an error
        }

        //if we have a folder tree, build the nodes.
        treeStatusLbl.Text = "Building the folder tree...";
        treeStatusLbl.Refresh();
        if (treeList.Count > 0)
        {
          TreeNode topNode = new TreeNode(topNodeName);
          topNode.Tag = new BOEFolderInfo(topNodeName, topNodeID, topNodeParentID);
          topNode.ImageIndex = 0;
          topNode.SelectedImageIndex = 0;
          //Build the folder tree
          foreach (BOEFolderInfo folderInfo in treeList)
          {
            topNode.Nodes.Add(BuildFolderNode(folderInfo));
          }
          //Add any reports that may be in the top node
          BuildRptNodes(topNodeID, topNode);
          topNode.ExpandAll();
          folderTreeView.Nodes.Add(topNode);
        }
      }
      finally
      {
        //Re-enable updates in the tree view which will automatically refresh with the new info
        folderTreeView.EndUpdate();
        //reset the cursor
        Cursor = Cursors.Default;
      }
    }

    /// <summary>
    /// Get the parent id of an object based on the object's id.
    /// </summary>
    /// <param name="objID">The id of the object</param>
    /// <returns>The id of the object's parent</returns>
    private Int32 GetParentID(int objID)
    {
      string query = "SI_PARENTID"
        + " From CI_INFOOBJECTS "
        + " Where SI_ID=" + objID.ToString();
      Int32 result = 0;
      try
      {
        //"using" ensures that the object is implicitly disposed when it goes out of scope
        IInfoObjects infoObjects = _BOEInfoStore.Query(query);
        
          if (infoObjects.Count > 0)
          {
            result = infoObjects[1].ParentID;
          }
        
      }
      catch
      {
        //eat the error - it will happen when we're looking at a top level folder
      }
      return result;
    }

    /// <summary>
    /// Build a list of folder info, including subfolders, based
    /// on the name of the top folder in the tree
    /// </summary>
    /// <param name="parentName"> Name of the starting folder</param>
    /// <returns>A list of BOEFolderInfo objects with information about the folders</returns>
    private List<BOEFolderInfo> GetFolderTree(string parentName)
    {
      int startFolder = GetIDFromName(parentName);
      List<BOEFolderInfo> result = GetFolderTree(startFolder);
      return result;
    }

    /// <summary>
    /// Build an list of folder info, including subfolders, based
    /// on the id of the top folder in the tree.  This is a recursive
    /// procedure that calls itself as it is "walking down" the folder tree.
    /// </summary>
    /// <param name="parentID"> ID of the starting folder</param>
    /// <returns>A list of BOEFolderInfo objects with information about the folders</returns>
    private List<BOEFolderInfo> GetFolderTree(int parentID)
    {
      List<BOEFolderInfo> folderTree = new List<BOEFolderInfo>();
      string query = "Select SI_NAME, SI_ID, SI_PARENTID, SI_PATH"
        + " From CI_INFOOBJECTS "
        + " Where SI_KIND='Folder'"
        + " And SI_PARENT_FOLDER=" + parentID.ToString()
        + " Order By SI_NAME";

      IInfoObjects infoObjects = _BOEInfoStore.Query(query);
      
        IInfoObject folder;
        BOEFolderInfo folderInfo;
        for (int i = 1; i <= infoObjects.Count; i++)
        {
          folder = (IInfoObject)infoObjects[i];
          folderInfo = new BOEFolderInfo(folder);
          folderInfo.SubFolders = GetFolderTree(folderInfo.ID);
          folderTree.Add(folderInfo);
        }
     
      return folderTree;
    }

    /// <summary>
    /// Recursive method to build the folder tree
    /// </summary>
    /// <param name="folderInfo">Information about the folder that is currently being processed</param>
    /// <returns>A TreeNode containing information about the specified folder, its subfolders and reports</returns>
    private TreeNode BuildFolderNode(BOEFolderInfo folderInfo)
    {
      TreeNode newNode;
      newNode = new TreeNode(folderInfo.FolderName);
      newNode.Tag = folderInfo;
      newNode.ImageIndex = 0;
      newNode.SelectedImageIndex = 0;

      foreach (BOEFolderInfo subFolder in folderInfo.SubFolders)
      {
        newNode.Nodes.Add(BuildFolderNode(subFolder));
      }
      BuildRptNodes(folderInfo.ID, newNode);
      return newNode;
    }

    /// <summary>
    /// Gets a list of the reports in a folder and adds them as nodes in the tree view.
    /// </summary>
    /// <param name="folderID">The folder we're currently processing</param>
    /// <param name="node">The TreeNode we're adding reports to</param>
    private void BuildRptNodes(Int32 folderID, TreeNode node)
    {
      List<BOERptInfo> rptList = GetReportsInFolder(folderID);
      TreeNode rptNode;
      foreach (BOERptInfo rptInfo in rptList)
      {
        rptNode = new TreeNode(rptInfo.RptName);
        rptNode.Tag = rptInfo;
        rptNode.ImageIndex = 1;
        rptNode.SelectedImageIndex = 1;
        node.Nodes.Add(rptNode);
      }
    }
    #endregion tree_view_methods

  }
}